-- Settings
--
-- This is the code for handling defaults and saving/loading settings

tbx.settings = {}

tbx.settings.settings = {}
tbx.settings.defaults = {
  windowPosX = 0,
  windowPosY = 0,
}

function tbx.settings.main()
-- Load/save settings event listeners
  table.insert(Event.Addon.SavedVariables.Load.End, {tbx.settings.load, "tbx", "Loaded settings"})
  table.insert(Event.Addon.SavedVariables.Save.Begin, {tbx.settings.save, "tbx", "Saved settings"})
end

-- Load the settings into the settings table
function tbx.settings.load(addon)
  if tbx_Settings ~= nil then tbx.settings.settings = tbx_Settings end
  tbx.dbg("Settings loaded")
end

--Save the settings table
function tbx.settings.save(addon)
  tbx_Settings = tbx.settings.settings
  tbx.dbg("Settings saved")
end


-- Get a specific setting's value
-- Params:
--   setting - the name of the setting to return
-- Returns:
--   The setting's value, if present. Otherwise the default setting's value.
--   If the setting is not found in either table, returns nil.
function tbx.settings.get(setting)
  if tbx.settings.settings[setting] ~= nil then
    return tbx.settings.settings[setting]
  elseif tbx.settings.defaults[setting] ~= nil then
    return tbx.settings.defaults[setting]
  else
    return nil
  end
end

-- Set a specific setting
-- Params:
--   setting - the name of the setting to set/modify the value of
--   value - the new value to be assigned to the setting
-- Returns:
--   The setting's new value.
function tbx.settings.set(setting, value)
  if tbx.settings.defaults[setting] == value then
    tbx.settings.settings[setting] = nil
  else
    tbx.settings.settings[setting] = value
  end
  
  return tbx.settings.settings[setting]
end

tbx.settings.main()