-- UI Handling
--------------

tbx.ui = {}

tbx.ui.context = UI.CreateContext("tbx")
tbx.ui.window = nil
tbx.ui.active = false

tbx.ui.needsReset = true

-- Window settings constants
local toolboxWindowSettings = {
  TITLE = string.format("Dimension Toolbox (v %.1f)", tbx.version),
  WIDTH = 700,
  HEIGHT = 690,
  CLOSABLE = true,
  MOVABLE = true,
}

-- Show the toolbox window
function tbx.ui.showToolboxWindow()
  tbx.constructions = tbx.settings.get("savedConstructions") or {}
  if tbx.ui.window == nil then
    tbx.ui.window = tbx.buildToolboxWindow()
    tbx.dbg("Toolbox window created")
  else
    tbx.ui.window:SetVisible(true)
    tbx.dbg("Toolbox window visible")
  end
  tbx.ui.active = true
  tbx.updateSelection()
end

-- Hide the toolbox window
function tbx.ui.hideToolboxWindow()
  tbx.ui.window:SetVisible(false)
  tbx.ui.window = nil
  tbx.constructions = {}
  tbx.ui.active = false
  tbx.dbg("Toolbox window destroyed")
end

-- Toggle the toolbox window
function tbx.ui.toggleToolboxWindow()
  if tbx.ui.active then tbx.ui.hideToolboxWindow()
  else tbx.ui.showToolboxWindow() end
end

-- Build the toolbox window
function tbx.buildToolboxWindow()
  local x = tbx.settings.get("windowPosX")
  local y = tbx.settings.get("windowPosY")
  window = tbx.ui.createWindow("ToolboxWindow",
                               toolboxWindowSettings.TITLE,
                               toolboxWindowSettings.WIDTH,
                               toolboxWindowSettings.HEIGHT,
                               x, y,
                               toolboxWindowSettings.CLOSABLE,
                               toolboxWindowSettings.MOVABLE,
                               tbx.ui.toolboxWindowClosed,
                               tbx.ui.toolboxWindowMoved
                              )
  --------------
  --ITEM DETAILS
  --------------
  window.itemDetails = tbx.ui.createFrame("itemDetails", window, 20, 65, window:GetWidth()/2-40, 120)
  --window.itemDetails:SetBackgroundColor(1, 0, 0, 0.5) -- debug
  
  window.itemDetails.icon = tbx.ui.createIcon("itemDetailsIcon", window.itemDetails, nil, 5, 5, 32)
  window.itemDetails.name = tbx.ui.createText("itemDetailsName", window.itemDetails, 37, 5, "Nothing selected", 22)
  
  window.itemDetails.xLabel = tbx.ui.createText("itemDetailsXLabel", window.itemDetails, 5, 42, "X:", 14, {1, 0, 0, 1})
  window.itemDetails.yLabel = tbx.ui.createText("itemDetailsYLabel", window.itemDetails, 5, 58, "Y:", 14, {0, 1, 0, 1})
  window.itemDetails.zLabel = tbx.ui.createText("itemDetailsZLabel", window.itemDetails, 5, 74, "Z:", 14, {0, 1, 1, 1})
  
  window.itemDetails.x = tbx.ui.createText("itemDetailsX", window.itemDetails, 25, 42, "-", 14)
  window.itemDetails.y = tbx.ui.createText("itemDetailsY", window.itemDetails, 25, 58, "-", 14)
  window.itemDetails.z = tbx.ui.createText("itemDetailsZ", window.itemDetails, 25, 74, "-", 14)
  
  window.itemDetails.yawLabel = tbx.ui.createText("itemDetailsYawLabel", window.itemDetails, 175, 42, "Yaw:", 14, {0, 1, 0, 1})
  window.itemDetails.pitchLabel = tbx.ui.createText("itemDetailsPitchLabel", window.itemDetails, 175, 58, "Pitch:", 14, {1, 0, 0, 1})
  window.itemDetails.rollLabel = tbx.ui.createText("itemDetailsRollLabel", window.itemDetails, 175, 74, "Roll:", 14, {0, 1, 1, 1})
  
  window.itemDetails.yaw = tbx.ui.createText("itemDetailsYaw", window.itemDetails, 217, 42, "-", 14)
  window.itemDetails.pitch = tbx.ui.createText("itemDetailsPitch", window.itemDetails, 217, 58, "-", 14)
  window.itemDetails.roll = tbx.ui.createText("itemDetailsRoll", window.itemDetails, 217, 74, "-", 14)
  
  window.itemDetails.scaleLabel = tbx.ui.createText("itemDetailsScaleLabel", window.itemDetails, 90, 90, "Scale:", 14)
  window.itemDetails.scale = tbx.ui.createText("itemDetailsScale", window.itemDetails, 132, 90, "-", 14)
  
  ---
  window.divider1 = tbx.ui.createTexture("divider1", window, "tbx", "textures/divider.png", 20, 180, window:GetWidth()/2-40)
  ------------------
  -- MODIFY POSITION
  ------------------
  window.modifyPosition = tbx.ui.createFrame("modifyPosition", window, 20, 205, window:GetWidth()/2-40, 80)
  --window.modifyPosition:SetBackgroundColor(1, 0, 0, 0.5) -- debug
  
  window.modifyPosition.xLabel = tbx.ui.createText("modifyPositionXLabel", window.modifyPosition, 5, 5, "X:", 14, {1, 0, 0, 1})
  window.modifyPosition.yLabel = tbx.ui.createText("modifyPositionYLabel", window.modifyPosition, 90, 5, "Y:", 14, {0, 1, 0, 1})
  window.modifyPosition.zLabel = tbx.ui.createText("modifyPositionZLabel", window.modifyPosition, 175, 5, "Z:", 14, {0, 1, 1, 1})
  
  window.modifyPosition.x = tbx.ui.createTextfield("modifyPositionX", window.modifyPosition, 25, 5, 50)
  window.modifyPosition.y = tbx.ui.createTextfield("modifyPositionY", window.modifyPosition, 110, 5, 50)
  window.modifyPosition.z = tbx.ui.createTextfield("modifyPositionZ", window.modifyPosition, 195, 5, 50)
  
  window.modifyPosition.modeAbs = tbx.ui.createCheckbox("modifyPositionModeAbs", window.modifyPosition, 5, 27, "Absolute", true, nil, tbx.ui.modifyPositionModeAbsChanged)
  window.modifyPosition.modeRel = tbx.ui.createCheckbox("modifyPositionModeRel", window.modifyPosition, 90, 27, "Relative", false, nil, tbx.ui.modifyPositionModeRelChanged)
  
  window.modifyPosition.moveAsGrp = tbx.ui.createCheckbox("modifyPositionMoveAsGrp", window.modifyPosition, 175, 27, "Move as group", true)
  window.modifyPosition.moveAsGrp:SetVisible(false)
  
  window.modifyPosition.changeBtn = tbx.ui.createButton("modifyPositionBtn", window.modifyPosition, 5, 43, nil, nil, "Move", nil, tbx.ui.modifyPositionButtonClicked)
  window.modifyPosition.resetBtn = tbx.ui.createButton("modifyPositionResetBtn", window.modifyPosition, 150, 43, nil, nil, "Reset", nil, tbx.ui.modifyPositionResetButtonClicked)
  
  ---
  window.divider2 = tbx.ui.createTexture("divider2", window, "tbx", "textures/divider.png", 20, 280, window:GetWidth()/2-40)
  ------------------
  -- MODIFY ROTATION
  ------------------
  window.modifyRotation = tbx.ui.createFrame("modifyRotation", window, 20, 305, window:GetWidth()/2-40, 80)
  --window.modifyRotation:SetBackgroundColor(1, 0, 0, 0.5) -- debug
  
  window.modifyRotation.yawLabel = tbx.ui.createText("modifyRotationYawLabel", window.modifyRotation, 5, 5, "Yaw:", 14, {0, 1, 0, 1})
  window.modifyRotation.pitchLabel = tbx.ui.createText("modifyRotationPitchLabel", window.modifyRotation, 90, 5, "Pitch:", 14, {1, 0, 0, 1})
  window.modifyRotation.rollLabel = tbx.ui.createText("modifyRotationRollLabel", window.modifyRotation, 175, 5, "Roll:", 14, {0, 1, 1, 1})
  
  window.modifyRotation.yaw = tbx.ui.createTextfield("modifyRotationYaw", window.modifyRotation, 42, 5, 40)
  window.modifyRotation.pitch = tbx.ui.createTextfield("modifyRotationPitch", window.modifyRotation, 132, 5, 40)
  window.modifyRotation.roll = tbx.ui.createTextfield("modifyRotationRoll", window.modifyRotation, 212, 5, 40)
  
  window.modifyRotation.modeAbs = tbx.ui.createCheckbox("modifyRotationModeAbs", window.modifyRotation, 5, 27, "Absolute", true, nil, tbx.ui.modifyRotationModeAbsChanged)
  window.modifyRotation.modeRel = tbx.ui.createCheckbox("modifyRotationModeRel", window.modifyRotation, 90, 27, "Relative", false, nil, tbx.ui.modifyRotationModeRelChanged)
  
  window.modifyRotation.changeBtn = tbx.ui.createButton("modifyRotationBtn", window.modifyRotation, 5, 43, nil, nil, "Rotate", nil, tbx.ui.modifyRotationButtonClicked)
  window.modifyRotation.resetBtn = tbx.ui.createButton("modifyRotationResetBtn", window.modifyRotation, 150, 43, nil, nil, "Reset", nil, tbx.ui.modifyRotationResetButtonClicked)
  
  ---
  window.divider3 = tbx.ui.createTexture("divider3", window, "tbx", "textures/divider.png", 20, 380, window:GetWidth()/2-40)
  ---------------
  -- MODIFY SCALE
  ---------------
  window.modifyScale = tbx.ui.createFrame("modifyScale", window, 20, 405, window:GetWidth()/2-40, 80)
  --window.modifyScale:SetBackgroundColor(1, 0, 0, 0.5) -- debug
  
  window.modifyScale.label = tbx.ui.createText("modifyScaleLabel", window.modifyScale, 5, 5, "Scale:", 14)
  
  window.modifyScale.scale = tbx.ui.createTextfield("modifyScaleScale", window.modifyScale, 47, 5, 100)
  
  window.modifyScale.modeAbs = tbx.ui.createCheckbox("modifyScaleModeAbs", window.modifyScale, 5, 27, "Absolute", true, nil, tbx.ui.modifyScaleModeAbsChanged)
  window.modifyScale.modeRel = tbx.ui.createCheckbox("modifyScaleModeRel", window.modifyScale, 90, 27, "Relative", false, nil, tbx.ui.modifyScaleModeRelChanged)
  
  window.modifyScale.changeBtn = tbx.ui.createButton("modifyScaleBtn", window.modifyScale, 5, 43, nil, nil, "Scale", nil, tbx.ui.modifyScaleButtonClicked)
  window.modifyScale.resetBtn = tbx.ui.createButton("modifyScaleResetBtn", window.modifyScale, 150, 43, nil, nil, "Reset", nil, tbx.ui.modifyScaleResetButtonClicked)
  
  
  tbx.ui.updateItemDetails()
  
  ---
  window.divider4 = tbx.ui.createTexture("divider3", window, "tbx", "textures/divider.png", 20, 480, window:GetWidth()/2-40)
  ---------------
  -- COPY/PASTE
  ---------------
  window.copyPaste = tbx.ui.createFrame("copyPaste", window, 20, 505, window:GetWidth()/2-40, 150)
  --window.copyPaste:SetBackgroundColor(1, 0, 0, 0.5) -- debug
  
  window.copyPaste.copyBtn = tbx.ui.createButton("copyBtn", window.copyPaste, 5, 5, nil, nil, "Copy", nil, tbx.ui.copyButtonClicked)
  window.copyPaste.pasteBtn = tbx.ui.createButton("pasteBtn", window.copyPaste, 150, 5, nil, nil, "Paste", nil, tbx.ui.pasteButtonClicked)
  
  window.copyPaste.offsetLabel1 = tbx.ui.createText("copyPasteOffsetLabel1", window.copyPaste, 50, 35, "offset", 11)
  window.copyPaste.offsetLabel2 = tbx.ui.createText("copyPasteOffsetLabel3", window.copyPaste, 225, 35, "offset", 11)
  window.copyPaste.offsetLabel3 = tbx.ui.createText("copyPasteOffsetLabel4", window.copyPaste, 145, 109, "offset", 11)
  
  window.copyPaste.x = tbx.ui.createCheckbox("copyPasteX", window.copyPaste, 5, 50, "X", true, {1, 0, 0, 1})
  window.copyPaste.y = tbx.ui.createCheckbox("copyPasteY", window.copyPaste, 5, 73, "Y", true, {0, 1, 0, 1})
  window.copyPaste.z = tbx.ui.createCheckbox("copyPasteZ", window.copyPaste, 5, 96, "Z", true, {0, 1, 1, 1})

  window.copyPaste.yaw = tbx.ui.createCheckbox("copyPasteYaw", window.copyPaste, 175, 50, "Yaw", true, {0, 1, 0, 1})
  window.copyPaste.pitch = tbx.ui.createCheckbox("copyPastePitch", window.copyPaste, 175, 73, "Pitch", true, {1, 0, 0, 1})
  window.copyPaste.roll = tbx.ui.createCheckbox("copyPasteRoll", window.copyPaste, 175, 96, "Roll", true, {0, 1, 1, 1})
  
  window.copyPaste.scale = tbx.ui.createCheckbox("copyPasteScale", window.copyPaste, 90, 124, "Scale", true)
  
  window.copyPaste.xOffset = tbx.ui.createTextfield("copyPasteXOffset", window.copyPaste, 50, 50, 40)
  window.copyPaste.yOffset = tbx.ui.createTextfield("copyPasteYOffset", window.copyPaste, 50, 73, 40)
  window.copyPaste.zOffset = tbx.ui.createTextfield("copyPasteZOffset", window.copyPaste, 50, 96, 40)
  
  window.copyPaste.yawOffset = tbx.ui.createTextfield("copyPasteYawOffset", window.copyPaste, 225, 50, 40)
  window.copyPaste.pitchOffset = tbx.ui.createTextfield("copyPastePitchOffset", window.copyPaste, 225, 73, 40)
  window.copyPaste.rollOffset = tbx.ui.createTextfield("copyPasteRollOffset", window.copyPaste, 225, 96, 40)
  
  window.copyPaste.scaleOffset = tbx.ui.createTextfield("copyPasteScaleOffset", window.copyPaste, 145, 124, 40)
  
  window.copyPaste.multiplyOffsets = tbx.ui.createCheckbox("copyPasteMultiplyOffsets", window.copyPaste, 5, 143, "Offset each item", false)
  window.copyPaste.multiplyOffsets:SetVisible(false)
  
  
  ----------------
  -- CONSTRUCTIONS
  ----------------
  window.constructions = tbx.ui.createFrame("constructions", window, window:GetWidth()/2+20, 65, window:GetWidth()/2-40, 320)
  window.constructions:SetBackgroundColor(0, 0, 0, 0.5) -- debug
  
  window.constructions.label = tbx.ui.createText("constructionsLabel", window.constructions, 5, 5, "Save / Load Item Sets (Experimental)", 18)
  
  window.constructions.nameLabel = tbx.ui.createText("constructionsNameLabel", window.constructions, 5, 40, "Name:", 14)
  window.constructions.name = tbx.ui.createTextfield("constructionsName", window.constructions, 60, 40, 220)
  window.constructions.save = tbx.ui.createButton("constructionsSave", window.constructions, 5, 65, nil, nil, "Save Set", nil, tbx.ui.constructionSaveClicked)
  window.constructions.import = tbx.ui.createButton("constructionsImport", window.constructions, 150, 65, nil, nil, "Import Set", nil, tbx.ui.importButtonClicked)
  
  window.constructions.importLabel = tbx.ui.createText("constructionsImportLabel", window.constructions, 5, 95, "Import:", 11)
  window.constructions.importText = tbx.ui.createTextfield("constructionsImportText", window.constructions, 5, 112, window.constructions:GetWidth()-10)
  --[[window.constructions.importTextArea = UI.CreateFrame("SimpleTextArea", "constructionsImportText", window.constructions)
  window.constructions.importTextArea:SetText("")
  window.constructions.importTextArea:SetPoint("TOPLEFT", window.constructions, "TOPLEFT", 5, 112)
  window.constructions.importTextArea:SetBackgroundColor(0,0,0,0.7)
  window.constructions.importTextArea:SetHeight(100)
  window.constructions.importTextArea:SetWidth(window.constructions:GetWidth()-10)]]
  
  window.divider5 = tbx.ui.createTexture("divider5", window, "tbx", "textures/divider.png", window:GetWidth()/2+20, 200, window:GetWidth()/2-40)
  window.divider5:SetLayer(98)
  
  window.constructions.nameLabel2 = tbx.ui.createText("constructionsNameLabel2", window.constructions, 5, 160, "Name:", 14)
  window.constructions.nameLoad = UI.CreateFrame("SimpleSelect", "constructionsNameSelect", window.constructions)
  window.constructions.nameLoad:SetPoint("TOPLEFT", window.constructions, "TOPLEFT", 60, 160)
  window.constructions.nameLoad:SetLayer(99)
  window.constructions.nameLoad:SetItems(tbx.ui.loadConstructions())
  window.constructions.nameLoad:ResizeToFit()
  window.constructions.nameLoad:SetWidth(220)
  window.constructions.load = tbx.ui.createButton("constructionsLoad", window.constructions, 5, 185, nil, nil, "Load Set", nil, tbx.ui.constructionLoadClicked)
  window.constructions.remove = tbx.ui.createButton("constructionsRemove", window.constructions, 150, 185, nil, nil, "Remove Set", nil, tbx.ui.constructionRemoveClicked)
  window.constructions.printReqs = tbx.ui.createButton("constructionsPrintReqs", window.constructions, 5, 220, nil, nil, "Print Materials", nil, tbx.ui.constructionPrintMaterials)
  window.constructions.export = tbx.ui.createButton("constructionsExport", window.constructions, 150, 220, nil, nil, "Export Set", nil, tbx.ui.exportButtonClicked)
  
  window.constructions.exportLabel = tbx.ui.createText("constructionsExportLabel", window.constructions, 5, 250, "Export:", 11)
  window.constructions.exportText = tbx.ui.createTextfield("constructionsExportText", window.constructions, 5, 267, window.constructions:GetWidth()-10)
  --[[window.constructions.exportTextArea = UI.CreateFrame("SimpleTextArea", "constructionsExportText", window.constructions)
  window.constructions.exportTextArea:SetPoint("TOPLEFT", window.constructions, "TOPLEFT", 5, 347)
  window.constructions.exportTextArea:SetText("")
  window.constructions.exportTextArea:SetBackgroundColor(0,0,0,0.7)
  window.constructions.exportTextArea:SetHeight(100)
  window.constructions.exportTextArea:SetWidth(window.constructions:GetWidth()-10)]]
  
  window.constructions.loadAtOriginalLoc = tbx.ui.createCheckbox("constructionLoadAtOriginalLoc", window.constructions, 5, 290, "Load at original location", false)
  
  return window
end

-- POSITION CHECKBOXES / BUTTON HANDLERS
----------------------------------------
function tbx.ui.modifyPositionModeAbsChanged()
  if tbx.ui.window.modifyPosition.modeAbs:GetChecked() then
    tbx.ui.window.modifyPosition.modeRel:SetChecked(false)
  elseif not tbx.ui.window.modifyPosition.modeRel:GetChecked() then
    tbx.ui.window.modifyPosition.modeAbs:SetChecked(true)
  end
end

function tbx.ui.modifyPositionModeRelChanged()
  if tbx.ui.window.modifyPosition.modeRel:GetChecked() then
    tbx.ui.window.modifyPosition.modeAbs:SetChecked(false)
  elseif not tbx.ui.window.modifyPosition.modeAbs:GetChecked() then
    tbx.ui.window.modifyPosition.modeRel:SetChecked(true)
  end
end

function tbx.ui.modifyPositionButtonClicked()
  --TODO number check on text boxes
  tbx.setItemPositions(tbx.ui.window.modifyPosition.x:GetText(), --x
                      tbx.ui.window.modifyPosition.y:GetText(), --y
                      tbx.ui.window.modifyPosition.z:GetText(), --z
                      tbx.ui.window.modifyPosition.modeRel:GetChecked(), --abs/rel
                      tbx.ui.window.modifyPosition.moveAsGrp:GetChecked()) --groupmove
end

function tbx.ui.modifyPositionResetButtonClicked()
  --TODO number check on text boxes
  tbx.resetItemPositions()
end

-- ROTATION CHECKBOXES / BUTTON HANDLERS
----------------------------------------
function tbx.ui.modifyRotationModeAbsChanged()
  if tbx.ui.window.modifyRotation.modeAbs:GetChecked() then
    tbx.ui.window.modifyRotation.modeRel:SetChecked(false)
  elseif not tbx.ui.window.modifyRotation.modeRel:GetChecked() then
    tbx.ui.window.modifyRotation.modeAbs:SetChecked(true)
  end
end

function tbx.ui.modifyRotationModeRelChanged()
  if tbx.ui.window.modifyRotation.modeRel:GetChecked() then
    tbx.ui.window.modifyRotation.modeAbs:SetChecked(false)
  elseif not tbx.ui.window.modifyRotation.modeAbs:GetChecked() then
    tbx.ui.window.modifyRotation.modeRel:SetChecked(true)
  end
end

function tbx.ui.modifyRotationButtonClicked()
  --TODO number check on text boxes
  tbx.setItemRotations(tbx.ui.window.modifyRotation.yaw:GetText(), --yaw
                      tbx.ui.window.modifyRotation.pitch:GetText(), --pitch
                      tbx.ui.window.modifyRotation.roll:GetText(), --roll
                      tbx.ui.window.modifyRotation.modeRel:GetChecked()) --abs/rel
end

function tbx.ui.modifyRotationResetButtonClicked()
  --TODO number check on text boxes
  tbx.resetItemRotations()
end

-- SCALE CHECKBOXES / BUTTON HANDLERS
-------------------------------------
function tbx.ui.modifyScaleModeAbsChanged()
  if tbx.ui.window.modifyScale.modeAbs:GetChecked() then
    tbx.ui.window.modifyScale.modeRel:SetChecked(false)
  elseif not tbx.ui.window.modifyScale.modeRel:GetChecked() then
    tbx.ui.window.modifyScale.modeAbs:SetChecked(true)
  end
end

function tbx.ui.modifyScaleModeRelChanged()
  if tbx.ui.window.modifyScale.modeRel:GetChecked() then
    tbx.ui.window.modifyScale.modeAbs:SetChecked(false)
  elseif not tbx.ui.window.modifyScale.modeAbs:GetChecked() then
    tbx.ui.window.modifyScale.modeRel:SetChecked(true)
  end
end

function tbx.ui.modifyScaleButtonClicked()
  --TODO number check on text boxes
  tbx.setItemScales(tbx.ui.window.modifyScale.scale:GetText(),
                   tbx.ui.window.modifyScale.modeRel:GetChecked()) --abs/rel
end

function tbx.ui.modifyScaleResetButtonClicked()
  --TODO number check on text boxes
  tbx.resetItemScales()
end

-- COPY / PASTE BUTTON HANDLERS
-------------------------------
function tbx.ui.copyButtonClicked()
  tbx.copyItemAttributes()
end

function tbx.ui.pasteButtonClicked()
  tbx.pasteMultipleItemAttributes(tbx.ui.window.copyPaste.x:GetChecked(),
                          tbx.ui.window.copyPaste.y:GetChecked(),
                          tbx.ui.window.copyPaste.z:GetChecked(),
                          tbx.ui.window.copyPaste.yaw:GetChecked(),
                          tbx.ui.window.copyPaste.pitch:GetChecked(),
                          tbx.ui.window.copyPaste.roll:GetChecked(),
                          tbx.ui.window.copyPaste.scale:GetChecked(),
                          tbx.ui.window.copyPaste.xOffset:GetText(),
                          tbx.ui.window.copyPaste.yOffset:GetText(),
                          tbx.ui.window.copyPaste.zOffset:GetText(),
                          tbx.ui.window.copyPaste.yawOffset:GetText(),
                          tbx.ui.window.copyPaste.pitchOffset:GetText(),
                          tbx.ui.window.copyPaste.rollOffset:GetText(),
                          tbx.ui.window.copyPaste.scaleOffset:GetText(),
                          tbx.ui.window.copyPaste.multiplyOffsets:GetChecked())
end

-- CONSTRUCTION BUTTON HANDLERS ETC
-----------------------------------

function tbx.ui.constructionSaveClicked()
  tbx.saveGroupItemAttributes(tbx.ui.window.constructions.name:GetText())
  tbx.ui.refreshLoadSelect()
end

function tbx.ui.constructionLoadClicked()
  tbx.loadGroupItemAttributes(tbx.ui.window.constructions.nameLoad:GetSelectedItem(),
                              window.constructions.loadAtOriginalLoc:GetChecked())
end

function tbx.ui.constructionRemoveClicked()
  tbx.removeGroupItem(tbx.ui.window.constructions.nameLoad:GetSelectedItem())
  tbx.ui.refreshLoadSelect()
end

function tbx.ui.importButtonClicked()
  tbx.importItemSet(tbx.ui.window.constructions.name:GetText(),
                    tbx.ui.window.constructions.importText:GetText())
  tbx.ui.refreshLoadSelect()
end

function tbx.ui.exportButtonClicked()
  tbx.exportItemSet(tbx.ui.window.constructions.nameLoad:GetSelectedItem())
end

function tbx.ui.refreshLoadSelect()
  tbx.ui.window.constructions.nameLoad:SetItems(tbx.ui.loadConstructions())
  tbx.ui.window.constructions.nameLoad:ResizeToFit()
  tbx.ui.window.constructions.nameLoad:SetWidth(220)
end

function tbx.ui.constructionPrintMaterials()
  tbx.printShoppingList(tbx.ui.window.constructions.nameLoad:GetSelectedItem())
end

function tbx.ui.loadConstructions()
  if tbx.constructions == nil or tbx.constructions == {} then
    return {}
  else
    local items = {}
    for name, _ in pairs(tbx.constructions) do
      table.insert(items, name)
    end
    return items
  end
end





-- OTHER STUFF
--------------

function tbx.ui.updateItemDetails()
  if tbx.ui.window ~= nil then
    if #tbx.selectedItems > 0 then -- Atleast one item selected
      if #tbx.selectedItems > 1 then -- Multiple items selected
        tbx.ui.window.itemDetails.icon:SetTexture("tbx", "textures/multiple.png")
        tbx.ui.window.itemDetails.name:SetText("Multiple items selected")
        local cp = tbx.getCentralPoint(tbx.selectedItems)
        tbx.ui.window.itemDetails.x:SetText(tostring(tbx.round(cp.x,4)))
        tbx.ui.window.itemDetails.y:SetText(tostring(tbx.round(cp.y,4)))
        tbx.ui.window.itemDetails.z:SetText(tostring(tbx.round(cp.z,4)))
        tbx.ui.window.itemDetails.yaw:SetText("-")
        tbx.ui.window.itemDetails.pitch:SetText("-")
        tbx.ui.window.itemDetails.roll:SetText("-")
        tbx.ui.window.itemDetails.scale:SetText("-")
        tbx.ui.window.modifyPosition.moveAsGrp:SetVisible(true)
        tbx.ui.window.copyPaste.multiplyOffsets:SetVisible(true)
      else -- Exactly one item selected
        if tbx.selectedItems[1].icon == "" then
          tbx.ui.window.itemDetails.icon:SetTexture("tbx", "textures/default.png")
        else
          tbx.ui.window.itemDetails.icon:SetTexture("Rift", tbx.selectedItems[1].icon)
        end
        tbx.ui.window.itemDetails.name:SetText(tostring(tbx.selectedItems[1].name))
        tbx.ui.window.itemDetails.x:SetText(tostring(tbx.round(tbx.selectedItems[1].coordX,4)))
        tbx.ui.window.itemDetails.y:SetText(tostring(tbx.round(tbx.selectedItems[1].coordY,4)))
        tbx.ui.window.itemDetails.z:SetText(tostring(tbx.round(tbx.selectedItems[1].coordZ,4)))
        tbx.ui.window.itemDetails.yaw:SetText(tostring(tbx.round(tbx.toDegrees(tbx.selectedItems[1].yaw),4)))
        tbx.ui.window.itemDetails.pitch:SetText(tostring(tbx.round(tbx.toDegrees(tbx.selectedItems[1].pitch),4)))
        tbx.ui.window.itemDetails.roll:SetText(tostring(tbx.round(tbx.toDegrees(tbx.selectedItems[1].roll),4)))
        tbx.ui.window.itemDetails.scale:SetText(tostring(tbx.round(tbx.selectedItems[1].scale,4)))
        tbx.ui.window.modifyPosition.moveAsGrp:SetVisible(false)
        tbx.ui.window.copyPaste.multiplyOffsets:SetVisible(false)
      end
    elseif tbx.ui.needsReset then -- no item selected
      tbx.ui.window.itemDetails.icon:SetTexture("tbx", "textures/blank.png")
      tbx.ui.window.itemDetails.name:SetText("Nothing selected")
      tbx.ui.window.itemDetails.x:SetText("-")
      tbx.ui.window.itemDetails.y:SetText("-")
      tbx.ui.window.itemDetails.z:SetText("-")
      tbx.ui.window.itemDetails.yaw:SetText("-")
      tbx.ui.window.itemDetails.pitch:SetText("-")
      tbx.ui.window.itemDetails.roll:SetText("-")
      tbx.ui.window.itemDetails.scale:SetText("-")
      tbx.ui.window.modifyPosition.moveAsGrp:SetVisible(false)
      tbx.ui.window.copyPaste.multiplyOffsets:SetVisible(false)
      tbx.ui.needsReset = false
    end
  end
end

-- Called when the window has been moved
function tbx.ui.toolboxWindowMoved()
  tbx.settings.set("windowPosX", window:GetLeft())
  tbx.settings.set("windowPosY", window:GetTop())
end

--Called when the window has been closed
function tbx.ui.toolboxWindowClosed()
  tbx.ui.hideToolboxWindow()
end











--------------------------------------------------------------------------------
-- FUNCTIONS FOR CREATING BASIC UI ELEMENTS ------------------------------------
--------------------------------------------------------------------------------

-- Create a text frame
function tbx.ui.createText(name, parent, x, y, text, fontsize, color)
  local textFrame = UI.CreateFrame("Text", name, parent)
  textFrame:SetFontSize(fontsize)
  textFrame:SetText(text)
  textFrame:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y)
  if color ~= nil then textFrame:SetFontColor(color[1], color[2], color[3], color[4]) end
	return textFrame
end

-- create icon
function tbx.ui.createIcon(name, parent, texture, x, y, size)
  local textureFrame = UI.CreateFrame("Texture", name, parent)
  if texture ~= nil then textureFrame:SetTexture("Rift", texture) end
  textureFrame:SetWidth(size)
  textureFrame:SetHeight(size)
  textureFrame:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y)
	return textureFrame
end

-- create texture
function tbx.ui.createTexture(name, parent, source, texture, x, y, width, height)
  local textureFrame = UI.CreateFrame("Texture", name, parent)
  if texture ~= nil then textureFrame:SetTexture(source, texture) end
  if width ~= nil then textureFrame:SetWidth(width) end
  if height ~= nil then textureFrame:SetHeight(height) end
  textureFrame:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y)
	return textureFrame
end

function tbx.ui.createTextfield(name, parent, x, y, width, text)
  local textfield = UI.CreateFrame("RiftTextfield", name, parent)
  textfield:SetWidth(width)
  textfield:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y)
  textfield:SetBackgroundColor(0,0,0,0.75)
  if text ~= nil then textfield:SetText(text)
  else textfield:SetText("") end
  return textfield
end

function tbx.ui.createCheckbox(name, parent, x, y, text, checked, fontColor, checkChangedCallback)
  local checkbox = UI.CreateFrame("RiftCheckbox", name, parent)
  checkbox:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y)
  if checked ~= nil then checkbox:SetChecked(checked) end
  
  checkbox.textFrame = tbx.ui.createText(name .. "TextFrame", checkbox, 15, -2, text, 14, fontColor)
  
  function checkbox.textFrame.Event:LeftClick()
    checkbox:SetChecked(not checkbox:GetChecked())
  end
  
  function checkbox.Event:CheckboxChange()
    if checkChangedCallback ~= nil then checkChangedCallback() end
  end
  return checkbox
end

-- Create a frame with a set height and width
function tbx.ui.createFrame(name, parent, x, y, width, height, color)
	local frame = UI.CreateFrame("Frame", name, parent)
	frame:SetHeight(height)
	frame:SetWidth(width)
  frame:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y)
  if color ~= nil then frame:SetBackgroundColor(color[1], color[2], color[3], color[4]) end
	return frame
end

-- Create a window with a set height, width and title
-- 'closeable' indicates whether the window should have a close button
-- 'closeCallback' specifies a function to be called when the close button fires
-- Otherwise it defaults to making window invisible
function tbx.ui.createWindow(name, title, width, height, x, y, closable, movable, closeCallback, moveCallback)
  local window = UI.CreateFrame("RiftWindow", name, tbx.ui.context)
	window:SetHeight(height)
	window:SetWidth(width)
  window:SetTitle(title)
  window:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
  
  if closable then
    window.closeBtn = tbx.ui.createButton(name .. "CloseBtn", window, window:GetWidth()-43, 16,0, 0, nil, "close", closeCallback)
    --window.closeBtn:SetPoint("TOPRIGHT", window, "TOPRIGHT", -8, 16)
  end
  
  --if movable then tbx.ui.makeMovable(window, window) end
  if movable then
    window.moveFrame = UI.CreateFrame("Texture", name .. "WindowMover", window) --todo change this to frame
    window.moveFrame:SetPoint("TOPLEFT", window, "TOPLEFT", 55, 13)
    window.moveFrame:SetPoint("BOTTOMRIGHT", window, "TOPRIGHT", -55, 45)
    --window.moveFrame:SetBackgroundColor(1,0,0, 0.5) -- for debugging
    
    local dragging = false
    
    function window.moveFrame.Event:LeftDown()
      dragging = true
      mouse = Inspect.Mouse()
      dragStartX = mouse.x - window:GetLeft()
      dragStartY = mouse.y - window:GetTop()
    end
    
    function window.moveFrame.Event:LeftUp()
      dragging = false
      if moveCallback ~= nil then moveCallback() end
    end
    
    function window.moveFrame.Event:MouseMove(x, y)
      if dragging then
        window:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x - dragStartX, y - dragStartY)
      end
    end
  end
  
	return window
end

-- Create a button with a set title, height and width
-- If skin is set to "close", button will be a close button (red X)
function tbx.ui.createButton(name, parent, x, y, width, height, text, skin, clickCallback)
  local button = UI.CreateFrame("RiftButton", name, parent)
  
  if skin ~= nil and skin == "close" then
    button:SetSkin("close")
  else
    if height ~= nil then button:SetHeight(height) end
    if width ~= nil then button:SetWidth(width) end
    if text ~= nil then button:SetText(text) end
  end
  
  button:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y)
  
  function button.Event:LeftPress()
    if clickCallback ~= nil then clickCallback() end
  end
  
  return button
end